package edu.travel.commodity.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.commodity.entity.ShopParameters;
import edu.travel.commodity.entity.ShopProductParameters;
import edu.travel.commodity.mapper.ShopParametersMapper;
import edu.travel.commodity.service.ShopParametersService;
import edu.travel.commodity.service.ShopProductParametersService;
import edu.travel.commodity.utils.PageUtil;
import edu.travel.remote.dto.ParametersDto;
import edu.travel.remote.dto.ProductSpecDto;
import edu.travel.remote.vo.ParametersVo;
import edu.travel.remote.vo.ShopParametersVo;
import edu.travel.service.SysServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class ShopParametersServiceImpl extends SysServiceImpl<ShopParametersMapper, ShopParameters> implements ShopParametersService {
    @Autowired
    private ShopProductParametersService shopProductParametersService;

    @Override
    public Page<ShopParametersVo> getShopParameters(ProductSpecDto param) {
        LambdaQueryWrapper<ShopProductParameters> query = Wrappers.<ShopProductParameters>lambdaQuery().eq(ShopProductParameters::getProductId, param.getProductId());

        Page<ShopProductParameters> page = shopProductParametersService.page(new Page<>(param.getCurrentPage(), param.getPageSize()), query);
        //分页获取参数值和参数ID
        Page<ShopParametersVo> pageEntity = PageUtil.toPageEntity(page, ShopParametersVo.class);
        List<ShopParametersVo> list = pageEntity.getRecords();
        if(ObjectUtil.isEmpty(list)){
            return pageEntity;
        }
        //参数IDS
        Set<String> collect = list.stream().map(ShopParametersVo::getParametersId).collect(Collectors.toSet());
        //参数值
        List<ShopParameters> parameters = lambdaQuery().in(ShopParameters::getId, collect).list();
        HashMap<String, ShopParameters> map = new HashMap<>();
        for (ShopParameters parameter : parameters) {
            map.put(parameter.getId().toString(), parameter);
        }
        //封装返回结果
        for (ShopParametersVo shopParametersVo : list) {
            ShopParameters shopParameters = map.get(shopParametersVo.getParametersId());
            if(ObjectUtil.isNotEmpty(shopParameters)){
                shopParametersVo.setParametersName(shopParameters.getParametersName());
                shopParametersVo.setIsNecessary(shopParameters.getIsNecessary());
            }
        }
        return pageEntity;
    }

    @Override
    public Page<ParametersVo> getAllParameters(ParametersDto param) {
        LambdaQueryWrapper<ShopParameters> query = Wrappers.<ShopParameters>lambdaQuery().
                eq(ShopParameters::getCountryId, param.getCountryId());
        if(param.getIsNecessary().equals("0")){
            query.eq(ShopParameters::getIsNecessary, param.getIsNecessary());
        }
        Page<ShopParameters> page = this.page(new Page<>(param.getCurrentPage(), param.getPageSize()),query);
        return PageUtil.toPageEntity(page, ParametersVo.class);
    }

    @Override
    public List<ShopParameters> getAllList() {
        return getListLink(null);
    }


}
